import dataclasses
import datetime as dt
import io
import logging
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

import pandas as pd
import psycopg2
import pytz
import sqlalchemy

from cloud_validol.loader.lib import pg

logger = logging.getLogger(__name__)


@dataclasses.dataclass
class DerivativeConfig:
    name: str
    source: str
    table_name: str
    platform_code_col: str
    derivative_name_col: str
    date_col: str
    date_format: str
    data_cols: Dict[str, str]
    report_type: str
    initial_from_year: int
    download_config: Any = None


@dataclasses.dataclass
class UpdateInterval:
    load_initial: bool
    years_to_load: List[int]


def get_interval(
    engine: sqlalchemy.engine.base.Engine, config: DerivativeConfig
) -> Optional[UpdateInterval]:
    df = pd.read_sql(
        f'''
            SELECT 
                MAX(DATE(data.event_dttm)) AS last_event_dt
            FROM validol_internal.{config.table_name} AS data
            INNER JOIN validol_internal.cot_derivatives_info AS info
                ON data.cot_derivatives_info_id = info.id
            INNER JOIN validol_internal.cot_derivatives_platform AS platform
                ON info.cot_derivatives_platform_id = platform.id
            WHERE data.report_type = %(report_type)s AND platform.source = %(platform_source)s
        ''',
        engine,
        params={'report_type': config.report_type, 'platform_source': config.source},
    )

    last_event_dt = df.iloc[0].last_event_dt
    today = dt.date.today()

    if last_event_dt is None:
        logger.info('No data for %s, downloading from scratch', config.name)

        return UpdateInterval(
            load_initial=True,
            years_to_load=list(range(config.initial_from_year, today.year + 1)),
        )

    if last_event_dt >= today:
        logger.info('Data for %s is already up-to-date', config.name)

        return None

    logger.info(
        '%s is subject to update, downloading from %s', config.name, last_event_dt.year
    )

    return UpdateInterval(
        load_initial=False,
        years_to_load=list(range(last_event_dt.year, today.year + 1)),
    )


def process_raw_dataframe(
    config: DerivativeConfig,
    date_format: str,
    raw_df: pd.DataFrame,
) -> pd.DataFrame:
    usecols = [
        config.platform_code_col,
        config.derivative_name_col,
        config.date_col,
    ] + list(config.data_cols)
    raw_df = raw_df[usecols]

    raw_df = raw_df.assign(
        **{
            config.date_col: raw_df[config.date_col].map(
                lambda x: dt.datetime.strptime(x, date_format).replace(tzinfo=pytz.UTC)
            )
        }
    )

    raw_df = raw_df.rename(
        columns={
            **config.data_cols,
            **{
                config.platform_code_col: 'platform_code',
                config.date_col: 'event_dttm',
            },
        }
    )

    raw_df.platform_code = [x.strip() for x in raw_df.platform_code]

    platform_names = []
    derivative_names = []
    for derivative_dash_platform in raw_df[config.derivative_name_col]:
        derivative_name, platform_name = derivative_dash_platform.rsplit('-', 1)
        platform_names.append(platform_name.strip())
        derivative_names.append(derivative_name.strip())

    raw_df['platform_name'] = platform_names
    raw_df['derivative_name'] = derivative_names

    del raw_df[config.derivative_name_col]

    return raw_df


def insert_platforms_derivatives(
    conn: psycopg2.extensions.connection, config: DerivativeConfig, df: pd.DataFrame
):
    derivatives = set()
    platforms = {}
    for _, row in df.iterrows():
        derivatives.add((row.platform_code, row.derivative_name))
        platforms[row.platform_code] = row.platform_name

    platform_codes, platform_names = zip(*platforms.items())

    with conn.cursor() as cursor:
        cursor.execute(
            '''
            INSERT INTO validol_internal.cot_derivatives_platform (source, code, name)
            SELECT %s, UNNEST(%s), UNNEST(%s)
            ON CONFLICT (source, code) DO UPDATE SET
                name = EXCLUDED.name
            RETURNING id
        ''',
            (config.source, list(platform_codes), list(platform_names)),
        )
        platform_ids = dict(zip(platform_codes, pg.extract_ids_from_cursor(cursor)))

        derivative_platform_ids, derivative_names = zip(
            *[
                (platform_ids[platform_code], derivative_name)
                for platform_code, derivative_name in derivatives
            ]
        )
        cursor.execute(
            '''
            INSERT INTO validol_internal.cot_derivatives_info (cot_derivatives_platform_id, name)
            SELECT UNNEST(%s), UNNEST(%s)
            ON CONFLICT (cot_derivatives_platform_id, name) DO UPDATE SET
                name = EXCLUDED.name
            RETURNING id
        ''',
            (list(derivative_platform_ids), list(derivative_names)),
        )

        derivative_ids = dict(zip(derivatives, pg.extract_ids_from_cursor(cursor)))

    conn.commit()

    df['cot_derivatives_info_id'] = [
        derivative_ids[row.platform_code, row.derivative_name]
        for _, row in df.iterrows()
    ]


def insert_data(
    engine: sqlalchemy.engine.base.Engine, config: DerivativeConfig, df: pd.DataFrame
):
    for column in ['platform_code', 'platform_name', 'derivative_name']:
        del df[column]

    df['report_type'] = config.report_type

    df.to_sql(
        config.table_name,
        engine,
        schema='validol_internal',
        index=False,
        if_exists='append',
        method=pg.insert_on_conflict_do_nothing,
        chunksize=10000,
    )



import logging

import click

from cloud_validol.loader.lib import pg
from cloud_validol.loader.reports import prices
from cloud_validol.loader.reports import monetary
from cloud_validol.loader.reports import moex
from cloud_validol.loader.reports import cftc
from cloud_validol.loader.reports import ice

logger = logging.getLogger(__name__)

UPDATE_SOURCES = {
    'prices': prices.update,
    'monetary': monetary.update,
    'moex': moex.update,
    'cftc': cftc.update,
    'ice': ice.update,
}


@click.command()
@click.option('--source', '-s', multiple=True, default=UPDATE_SOURCES.keys())
def main(source):
    logging.basicConfig(
        format='%(asctime)s %(levelname)s:%(message)s',
        level=logging.DEBUG,
        datefmt='[%Y-%m-%d %H:%M:%S]',
    )

    engine = pg.get_engine()

    with pg.get_connection() as conn:
        for s in source:
            if s in UPDATE_SOURCES:
                UPDATE_SOURCES[s](engine, conn)
            else:
                logger.error('Passed nonexistent source to cli: %s', s)


if __name__ == '__main__':
    main()



import copy
import dataclasses
import io
import logging
from typing import List

import pandas as pd
import psycopg2
import requests
import sqlalchemy

from cloud_validol.loader.lib import cot

logger = logging.getLogger(__name__)


@dataclasses.dataclass
class IceDownloadConfig:
    report_type_filter: str


def _make_derivative_configs() -> List[cot.DerivativeConfig]:
    ice_disaggregated_futures_only = cot.DerivativeConfig(
        name='ice_disaggregated_futures_only',
        source='ice',
        table_name='cot_disaggregated_data',
        platform_code_col='CFTC_Market_Code',
        derivative_name_col='Market_and_Exchange_Names',
        date_col='As_of_Date_Form_MM/DD/YYYY',
        date_format='%m/%d/%Y',
        data_cols={
            'Open_Interest_All': 'oi',
            'Prod_Merc_Positions_Long_All': 'pmpl',
            'Prod_Merc_Positions_Short_All': 'pmps',
            'Swap_Positions_Long_All': 'sdpl',
            'Swap_Positions_Short_All': 'sdps',
            'M_Money_Positions_Long_All': 'mmpl',
            'M_Money_Positions_Short_All': 'mmps',
            'Other_Rept_Positions_Long_All': 'orpl',
            'Other_Rept_Positions_Short_All': 'orps',
            'NonRept_Positions_Long_All': 'nrl',
            'NonRept_Positions_Short_All': 'nrs',
            'Conc_Gross_LE_4_TDR_Long_All': '4gl%',
            'Conc_Gross_LE_4_TDR_Short_All': '4gs%',
            'Conc_Gross_LE_8_TDR_Long_All': '8gl%',
            'Conc_Gross_LE_8_TDR_Short_All': '8gs%',
            'Conc_Net_LE_4_TDR_Long_All': '4l%',
            'Conc_Net_LE_4_TDR_Short_All': '4s%',
            'Conc_Net_LE_8_TDR_Long_All': '8l%',
            'Conc_Net_LE_8_TDR_Short_All': '8s%',
            'Swap_Positions_Spread_All': 'sdp_spr',
            'M_Money_Positions_Spread_All': 'mmp_spr',
            'Other_Rept_Positions_Spread_All': 'orp_spr',
        },
        initial_from_year=2011,
        report_type='futures_only',
        download_config=IceDownloadConfig(report_type_filter='FutOnly'),
    )

    ice_disaggregated_combined = copy.deepcopy(ice_disaggregated_futures_only)
    ice_disaggregated_combined.name = 'ice_disaggregated_combined'
    ice_disaggregated_combined.download_config.report_type_filter = 'Combined'
    ice_disaggregated_combined.report_type = 'combined'

    return [
        ice_disaggregated_futures_only,
        ice_disaggregated_combined,
    ]


def _download_doc(
    year: int,
) -> pd.DataFrame:
    logger.info('Downloading %s for ICE', year)

    response = requests.get(
        'https://www.theice.com/publicdocs/futures/COTHist{year}.csv'.format(year=year),
        headers={'User-Agent': 'Mozilla/5.0'},
    )

    df = pd.read_csv(io.StringIO(response.text))
    df = df.rename(
        columns={
            'Swap__Positions_Short_All': 'Swap_Positions_Short_All',
            'Swap__Positions_Spread_All': 'Swap_Positions_Spread_All',
        }
    )

    return df


def update(engine: sqlalchemy.engine.base.Engine, conn: psycopg2.extensions.connection):
    logger.info('Start updating ICE data')

    configs = _make_derivative_configs()

    years_to_load = []
    for config in configs:
        update_interval = cot.get_interval(engine, config)

        if update_interval is None:
            continue

        years_to_load.extend(update_interval.years_to_load)

    overall_dfs = []
    for year in set(years_to_load):
        overall_dfs.append(_download_doc(year))

    overall_df = pd.concat(overall_dfs)
    overall_df = overall_df.replace('#VALUE!', None)

    for config in configs:
        raw_df = overall_df[
            overall_df['FutOnly_or_Combined']
            == config.download_config.report_type_filter
        ]
        df = cot.process_raw_dataframe(config, config.date_format, raw_df)

        cot.insert_platforms_derivatives(conn, config, df)
        cot.insert_data(engine, config, df)

    logger.info('Finish updating ICE data')



import datetime as dt
import io
import logging
import time
from typing import Optional
from typing import Tuple

import pandas as pd
import psycopg2
import pytz
import requests
import sqlalchemy
import tqdm

from cloud_validol.loader.lib import interval_utils
from cloud_validol.loader.lib import pg

logger = logging.getLogger(__name__)

GLOBAL_FROM = dt.date(2012, 11, 1)
CONTRACT_TYPE_MAPPING = {'F': 'Futures', 'C': 'Option call', 'P': 'Option put'}
CSV_MAPPING = {
    'clients_in_long': 'lq',
    'clients_in_short': 'sq',
    'short_position': 's',
    'long_position': 'l',
}
PHYS_MAPPING = {0: 'u', 1: 'f'}


def _dt_serializer(date: dt.date) -> str:
    return date.strftime('%Y%m%d')


def _download_date(date: dt.date) -> Optional[pd.DataFrame]:
    response = requests.get(
        'https://www.moex.com/ru/derivatives/open-positions-csv.aspx',
        params={'d': _dt_serializer(date)},
        headers={'User-Agent': 'Mozilla/5.0'},
    )

    df = pd.read_csv(
        io.StringIO(response.text),
        parse_dates=['moment'],
        date_parser=lambda x: dt.datetime.fromisoformat(x).replace(tzinfo=pytz.UTC),
    )

    if df.empty:
        return None

    df = df.rename(columns={'moment': 'event_dttm', 'isin': 'code'})

    df.name = df.apply(
        lambda row: '{} ({})'.format(
            row['name'], CONTRACT_TYPE_MAPPING[row.contract_type]
        ),
        axis=1,
    )
    df.iz_fiz.fillna(0, inplace=True)

    keys = ['event_dttm', 'name']

    return pd.DataFrame(
        [
            {
                **{
                    '{}{}'.format(PHYS_MAPPING[row.iz_fiz], value): row[key]
                    for _, row in content.iterrows()
                    for key, value in CSV_MAPPING.items()
                },
                **dict(zip(keys, group)),
            }
            for group, content in df.groupby(keys)
        ]
    )


def _get_interval(
    engine: sqlalchemy.engine.base.Engine,
) -> Optional[Tuple[dt.date, dt.date]]:
    df = pd.read_sql(
        '''
        SELECT MAX(DATE(event_dttm)) AS last_event_dt
        FROM validol_internal.moex_derivatives_data
    ''',
        engine,
    )

    last_event_dt = df.iloc[0].last_event_dt

    return interval_utils.get_interval('moex', last_event_dt, GLOBAL_FROM)


def update(engine: sqlalchemy.engine.base.Engine, conn: psycopg2.extensions.connection):
    logger.info('Start updating moex data')

    interval = _get_interval(engine)
    if interval is None:
        return

    from_date, to_date = interval
    dfs = []
    for date in tqdm.tqdm(pd.date_range(from_date, to_date)):
        date_data = _download_date(date)
        if date_data is not None:
            dfs.append(_download_date(date))

        time.sleep(2)

    if not dfs:
        return

    result_df = pd.concat(dfs)

    unique_derivative_names = list(result_df.name.unique())
    with conn.cursor() as cursor:
        cursor.execute(
            '''
            INSERT INTO validol_internal.moex_derivatives_info (name)
            SELECT UNNEST(%s)
            ON CONFLICT (name) DO UPDATE SET
                name = EXCLUDED.name
            RETURNING id
        ''',
            (unique_derivative_names,),
        )

        derivative_name_map_id = dict(
            zip(unique_derivative_names, pg.extract_ids_from_cursor(cursor))
        )

    conn.commit()

    result_df['moex_derivatives_info_id'] = [
        derivative_name_map_id[name] for name in result_df['name']
    ]
    del result_df['name']

    result_df.to_sql(
        'moex_derivatives_data',
        engine,
        schema='validol_internal',
        index=False,
        if_exists='append',
        chunksize=10000,
    )

    logger.info('Finish updating moex data')



import datetime as dt
import logging
from typing import Optional
from typing import Tuple

logger = logging.getLogger(__name__)


def get_interval(
    interval_slug: str, last_event_dt: Optional[dt.date], global_from: dt.date
) -> Optional[Tuple[dt.date, dt.date]]:
    to_date = dt.date.today()

    if last_event_dt is None:
        logger.info('No data for %s, downloading from %s', interval_slug, global_from)

        return global_from, to_date

    from_date = last_event_dt + dt.timedelta(days=1)
    if from_date >= to_date:
        logger.info('Data for %s is already up-to-date', interval_slug)

        return None

    logger.info(
        '%s is subject to update, downloading from %s', interval_slug, from_date
    )

    return from_date, to_date



import datetime as dt
import io
import logging

import numpy as np
import pandas as pd
import psycopg2
import pytz
import requests
import sqlalchemy

logger = logging.getLogger(__name__)


def update(engine: sqlalchemy.engine.base.Engine, conn: psycopg2.extensions.connection):
    logger.info('Start updating stlouisfed data')

    dfs = []
    for graph_id, sensor in [('BOGMBASEW', 'MBase'), ('ASTDSL', 'TDebt')]:
        response = requests.get(
            url='https://fred.stlouisfed.org/graph/fredgraph.csv',
            params={
                'id': graph_id,
            },
            headers={'Host': 'fred.stlouisfed.org', 'User-Agent': 'Mozilla/5.0'},
        )

        df = pd.read_csv(io.StringIO(response.text))
        df = df.replace('.', np.nan).dropna()
        df['event_dttm'] = df['DATE'].map(
            lambda x: dt.datetime.fromisoformat(x).replace(tzinfo=pytz.UTC)
        )
        df['sensor'] = sensor
        df = df.rename(columns={graph_id: 'value'})
        del df['DATE']

        dfs.append(df)

    with conn.cursor() as cursor:
        cursor.execute('TRUNCATE TABLE validol_internal.fredgraph')
    conn.commit()

    pd.concat(dfs).to_sql(
        'fredgraph',
        engine,
        schema='validol_internal',
        index=False,
        if_exists='append',
        chunksize=10000,
    )

    logger.info('Finish updating stlouisfed data')



import os
import json
from typing import Any
from typing import Dict
from typing import Iterable
from typing import List
from typing import Optional
from typing import Tuple

import pandas as pd
import psycopg2
import sqlalchemy


SECDIST_PATH = '/etc/cloud_validol/secdist.json'


def get_conn_data(conn_data: Optional[Dict[str, str]] = None) -> Dict[str, str]:
    if conn_data is not None:
        return conn_data
    elif os.path.isfile(SECDIST_PATH):
        with open(SECDIST_PATH) as infile:
            data = json.load(infile)

        return data['postgresql']
    else:
        return os.environ


def get_connstr(conn_data: Dict[str, str]) -> str:
    user = conn_data['DATABASE_USER']
    password = conn_data['DATABASE_PASSWORD']
    dbname = conn_data['DATABASE_DB']
    dbhost = conn_data['DATABASE_HOST']

    return f'postgresql+psycopg2://{user}:{password}@{dbhost}/{dbname}'


def get_engine(
    conn_data: Optional[Dict[str, str]] = None
) -> sqlalchemy.engine.base.Engine:
    return sqlalchemy.create_engine(get_connstr(get_conn_data(conn_data)))


def get_connection(
    conn_data: Optional[Dict[str, str]] = None
) -> psycopg2.extensions.connection:
    conn_data = get_conn_data(conn_data)

    return psycopg2.connect(
        user=conn_data['DATABASE_USER'],
        password=conn_data['DATABASE_PASSWORD'],
        dbname=conn_data['DATABASE_DB'],
        host=conn_data['DATABASE_HOST'],
    )


def insert_on_conflict_do_nothing(
    table: pd.io.sql.SQLTable,
    conn: sqlalchemy.engine.base.Connection,
    keys: List[str],
    data_iter: Iterable[Iterable[Any]],
):
    data = [dict(zip(keys, row)) for row in data_iter]
    insert_stmt = sqlalchemy.dialects.postgresql.insert(
        table.table, values=data, bind=conn
    ).on_conflict_do_nothing()
    conn.execute(insert_stmt)


def extract_ids_from_cursor(cursor: Iterable[Tuple[int]]):
    return [id_ for id_, in cursor]



import copy
import dataclasses
import io
import logging
from typing import List
from typing import Optional
import zipfile

import pandas as pd
import psycopg2
import requests
import sqlalchemy

from cloud_validol.loader.lib import cot

logger = logging.getLogger(__name__)


@dataclasses.dataclass
class CftcDownloadConfig:
    year_download_url: str
    initial_download_url: Optional[str] = None
    initial_date_format: Optional[str] = None


def _make_derivative_configs() -> List[cot.DerivativeConfig]:
    cftc_date_format = '%Y-%m-%d'

    cftc_disaggregated_futures_only = cot.DerivativeConfig(
        name='cftc_disaggregated_futures_only',
        source='cftc',
        table_name='cot_disaggregated_data',
        platform_code_col='CFTC_Market_Code',
        derivative_name_col='Market_and_Exchange_Names',
        date_col='Report_Date_as_YYYY-MM-DD',
        date_format=cftc_date_format,
        data_cols={
            'Open_Interest_All': 'oi',
            'Prod_Merc_Positions_Long_All': 'pmpl',
            'Prod_Merc_Positions_Short_All': 'pmps',
            'Swap_Positions_Long_All': 'sdpl',
            'Swap__Positions_Short_All': 'sdps',
            'M_Money_Positions_Long_All': 'mmpl',
            'M_Money_Positions_Short_All': 'mmps',
            'Other_Rept_Positions_Long_All': 'orpl',
            'Other_Rept_Positions_Short_All': 'orps',
            'NonRept_Positions_Long_All': 'nrl',
            'NonRept_Positions_Short_All': 'nrs',
            'Conc_Gross_LE_4_TDR_Long_All': '4gl%',
            'Conc_Gross_LE_4_TDR_Short_All': '4gs%',
            'Conc_Gross_LE_8_TDR_Long_All': '8gl%',
            'Conc_Gross_LE_8_TDR_Short_All': '8gs%',
            'Conc_Net_LE_4_TDR_Long_All': '4l%',
            'Conc_Net_LE_4_TDR_Short_All': '4s%',
            'Conc_Net_LE_8_TDR_Long_All': '8l%',
            'Conc_Net_LE_8_TDR_Short_All': '8s%',
            'Swap__Positions_Spread_All': 'sdp_spr',
            'M_Money_Positions_Spread_All': 'mmp_spr',
            'Other_Rept_Positions_Spread_All': 'orp_spr',
        },
        report_type='futures_only',
        initial_from_year=2017,
        download_config=CftcDownloadConfig(
            year_download_url='http://www.cftc.gov/files/dea/history/fut_disagg_txt_{year}.zip',
            initial_download_url='http://www.cftc.gov/files/dea/history/fut_disagg_txt_hist_2006_2016.zip',
        ),
    )

    cftc_disaggregated_combined = copy.deepcopy(cftc_disaggregated_futures_only)
    cftc_disaggregated_combined.name = 'cftc_disaggregated_combined'
    cftc_disaggregated_combined.download_config.year_download_url = (
        'http://www.cftc.gov/files/dea/history/com_disagg_txt_{year}.zip'
    )
    cftc_disaggregated_combined.download_config.initial_download_url = (
        'http://www.cftc.gov/files/dea/history/com_disagg_txt_hist_2006_2016.zip'
    )
    cftc_disaggregated_combined.report_type = 'combined'

    cftc_financial_futures_only = cot.DerivativeConfig(
        name='cftc_financial_futures_only',
        source='cftc',
        table_name='cot_financial_futures_data',
        platform_code_col='CFTC_Market_Code',
        derivative_name_col='Market_and_Exchange_Names',
        date_col='Report_Date_as_YYYY-MM-DD',
        date_format=cftc_date_format,
        data_cols={
            'Open_Interest_All': 'oi',
            'Dealer_Positions_Long_All': 'dipl',
            'Dealer_Positions_Short_All': 'dips',
            'Dealer_Positions_Spread_All': 'dip_spr',
            'Asset_Mgr_Positions_Long_All': 'ampl',
            'Asset_Mgr_Positions_Short_All': 'amps',
            'Asset_Mgr_Positions_Spread_All': 'amp_spr',
            'Lev_Money_Positions_Long_All': 'lmpl',
            'Lev_Money_Positions_Short_All': 'lmps',
            'Lev_Money_Positions_Spread_All': 'lmp_spr',
            'Other_Rept_Positions_Long_All': 'orpl',
            'Other_Rept_Positions_Short_All': 'orps',
            'Other_Rept_Positions_Spread_All': 'orp_spr',
            'NonRept_Positions_Long_All': 'nrl',
            'NonRept_Positions_Short_All': 'nrs',
        },
        report_type='futures_only',
        initial_from_year=2017,
        download_config=CftcDownloadConfig(
            year_download_url='http://www.cftc.gov/files/dea/history/fut_fin_txt_{year}.zip',
            initial_download_url='http://www.cftc.gov/files/dea/history/fin_fut_txt_2006_2016.zip',
            initial_date_format='%m/%d/%Y 12:00:00 AM',
        ),
    )

    cftc_financial_combined = copy.deepcopy(cftc_financial_futures_only)
    cftc_financial_combined.name = 'cftc_financial_combined'
    cftc_financial_combined.download_config.year_download_url = (
        'http://www.cftc.gov/files/dea/history/com_fin_txt_{year}.zip'
    )
    cftc_financial_combined.download_config.initial_download_url = (
        'http://www.cftc.gov/files/dea/history/fin_com_txt_2006_2016.zip'
    )
    cftc_financial_combined.report_type = 'combined'

    cftc_futures_only = cot.DerivativeConfig(
        name='cftc_futures_only',
        source='cftc',
        table_name='cot_futures_only_data',
        platform_code_col='CFTC Market Code in Initials',
        derivative_name_col='Market and Exchange Names',
        date_col='As of Date in Form YYYY-MM-DD',
        date_format='%Y-%m-%d',
        data_cols={
            'Open Interest (All)': 'oi',
            'Noncommercial Positions-Long (All)': 'ncl',
            'Noncommercial Positions-Short (All)': 'ncs',
            'Commercial Positions-Long (All)': 'cl',
            'Commercial Positions-Short (All)': 'cs',
            'Nonreportable Positions-Long (All)': 'nrl',
            'Nonreportable Positions-Short (All)': 'nrs',
            'Concentration-Net LT =4 TDR-Long (All)': '4l%',
            'Concentration-Net LT =4 TDR-Short (All)': '4s%',
            'Concentration-Net LT =8 TDR-Long (All)': '8l%',
            'Concentration-Net LT =8 TDR-Short (All)': '8s%',
        },
        report_type='futures_only',
        initial_from_year=2017,
        download_config=CftcDownloadConfig(
            year_download_url='http://www.cftc.gov/files/dea/history/deacot{year}.zip',
            initial_download_url='http://www.cftc.gov/files/dea/history/deacot1986_2016.zip',
        ),
    )

    return [
        cftc_futures_only,
        cftc_disaggregated_futures_only,
        cftc_disaggregated_combined,
        cftc_financial_futures_only,
        cftc_financial_combined,
    ]


def _download_doc(
    config: cot.DerivativeConfig,
    url: str,
) -> pd.DataFrame:
    logger.info('Downloading %s for %s', url, config.name)

    response = requests.get(url, headers={'User-Agent': 'Mozilla/5.0'})

    with zipfile.ZipFile(io.BytesIO(response.content), 'r') as zip_file:
        path = zip_file.namelist()[0]
        csv_buff = zip_file.read(path).decode('utf-8')

    return pd.read_csv(io.StringIO(csv_buff))


def update(engine: sqlalchemy.engine.base.Engine, conn: psycopg2.extensions.connection):
    logger.info('Start updating CFTC data')

    for config in _make_derivative_configs():
        update_interval = cot.get_interval(engine, config)

        if update_interval is None:
            continue

        dfs = []
        if update_interval.load_initial:
            raw_df = _download_doc(
                config,
                config.download_config.initial_download_url,
            )
            dfs.append(
                cot.process_raw_dataframe(
                    config,
                    config.download_config.initial_date_format or config.date_format,
                    raw_df,
                )
            )

        for year in update_interval.years_to_load:
            raw_df = _download_doc(
                config,
                config.download_config.year_download_url.format(year=year),
            )
            dfs.append(cot.process_raw_dataframe(config, config.date_format, raw_df))

        df = pd.concat(dfs)

        cot.insert_platforms_derivatives(conn, config, df)
        cot.insert_data(engine, config, df)

    logger.info('Finish updating CFTC data')



import datetime as dt
import logging
from typing import Dict

import investpy
import pandas as pd
import psycopg2
import pytz
import sqlalchemy
import tqdm

from cloud_validol.loader.lib import interval_utils

logger = logging.getLogger(__name__)

GLOBAL_FROM = dt.date(2010, 1, 1)


def _dt_serializer(date: dt.date) -> str:
    return date.strftime('%d/%m/%Y')


def _get_intervals(engine: sqlalchemy.engine.base.Engine) -> Dict[int, Dict[str, str]]:
    df = pd.read_sql(
        '''
        SELECT 
            info.id AS info_id,
            info.currency_cross AS info_currency_cross,
            MAX(DATE(event_dttm)) AS last_event_dt
        FROM validol_internal.investing_prices_info AS info
        LEFT JOIN validol_internal.investing_prices_data AS data 
            ON data.investing_prices_info_id = info.id
        GROUP BY info.id
    ''',
        engine,
    )

    result = {}
    for _, row in df.iterrows():
        interval = interval_utils.get_interval(
            row.info_currency_cross, row.last_event_dt, GLOBAL_FROM
        )
        if interval is not None:
            from_date, to_date = interval
            result[row.info_id] = {
                'currency_cross': row.info_currency_cross,
                'from_date': _dt_serializer(from_date),
                'to_date': _dt_serializer(to_date),
            }

    return result


def update(engine: sqlalchemy.engine.base.Engine, conn: psycopg2.extensions.connection):
    logger.info('Start updating prices')

    intervals = _get_intervals(engine)

    for info_id, interval in tqdm.tqdm(intervals.items()):
        df = investpy.get_currency_cross_historical_data(**interval)
        df.index = df.index.map(lambda x: x.replace(tzinfo=pytz.UTC))
        del df['Currency']
        df = df.rename(
            columns={
                'Open': 'open_price',
                'High': 'high_price',
                'Low': 'low_price',
                'Close': 'close_price',
            }
        )
        df['investing_prices_info_id'] = info_id
        df.to_sql(
            'investing_prices_data',
            engine,
            schema='validol_internal',
            index=True,
            index_label='event_dttm',
            if_exists='append',
            chunksize=10000,
        )

    logger.info('Finish updating prices')
